const MAX_DIM = 4096;
const { createCanvas, loadImage } = require('canvas');

function ImageCompare() {
    this.mode = ['affine', 'perspective', 'tps', 'similarity', 'identity'];
    this.loggedMessages = [];
    this.logger = console.log;
    const regex = /inlier_fraction\s+([\d.]+)/g;

    async function getImageData(path) {
        const image = await loadImage(path);
        this.bcanvas = createCanvas(image.width, image.height);
        this.bctx = this.bcanvas.getContext('2d');
        const { width: cw, height: ch } = this.bcanvas;
        this.bctx.clearRect(0, 0, cw, ch);
        const iw = image.width, ih = image.height;

        if (iw < MAX_DIM && ih < MAX_DIM) {
            this.bcanvas.width = iw;
            this.bcanvas.height = ih;
            this.bctx.drawImage(image, 0, 0);
            return this.bctx.getImageData(0, 0, this.bcanvas.width, this.bcanvas.height);
        }

        const max_dim = iw > ih ? iw : ih;
        const ow = Math.round((iw / max_dim) * MAX_DIM),
            oh = Math.round((ih / max_dim) * MAX_DIM);
            
        this.bcanvas.width = ow;
        this.bcanvas.height = oh;
        this.bctx.drawImage(image, 0, 0, iw, ih, 0, 0, ow, oh);
        return this.bctx.getImageData(0, 0, ow, oh);
    }
    
    this.initialization = async () => {
        console.log = (message) => { // overwrite
            this.loggedMessages.push(message);
        };
        this.transform = await require('./app/js/transform');
    }

    this.matching = async (img1, img2, mode) => {
        let lastFraction = 0;
        const im1Data = await getImageData(img1);
        const im2Data = await getImageData(img2);
        this.transform.estimate_transform( im2Data.data, im2Data.height, im1Data.data, im1Data.height, mode );
        if (this.loggedMessages.length != 0) {
            let match;
            while ((match = regex.exec(this.loggedMessages.pop())) !== null) {
                lastFraction = parseFloat(match[1]) * 100;
            }
        }
        this.loggedMessages = [];
        return lastFraction;
    }

    this.destroy = () => {
        this.loggedMessages = [];
        console.log = this.logger;
    }
}

module.exports = new ImageCompare;