const MAX_DIM = 4096;
const fs = require('fs');
const { createCanvas, loadImage } = require('canvas');
const images = fs.readdirSync('./images').filter(item => item != '.DS_Store').map(item => './images/' + item)
                .sort(compareFileNames).reduce((resultArray, item, index) => { 
                    const chunkIndex = Math.floor(index / 2);
                    if(!resultArray[chunkIndex]) {
                        resultArray[chunkIndex] = [];
                    }
                    resultArray[chunkIndex].push(item);  
                    return resultArray;
                }, []);

function compareFileNames(a, b) {
    // Extract numeric parts from filenames
    const numA = parseInt(a.match(/\d+/)[0]);
    const numB = parseInt(b.match(/\d+/)[0]);
    
    // Compare numeric parts
    if (numA < numB) {
      return -1;
    } else if (numA > numB) {
      return 1;
    } else {
      // If numeric parts are equal, compare entire strings
      return a.localeCompare(b);
    }
}

async function getImageData (path) {
    const image = await loadImage(path);
    this.bcanvas = createCanvas(image.width, image.height);
    this.bctx = this.bcanvas.getContext('2d');
    const { width: cw, height: ch } = this.bcanvas;
    this.bctx.clearRect(0, 0, cw, ch);
    const iw = image.width, ih = image.height;

    if (iw < MAX_DIM && ih < MAX_DIM) {
        this.bcanvas.width = iw;
        this.bcanvas.height = ih;
        this.bctx.drawImage(image, 0, 0);
        return this.bctx.getImageData(0, 0, this.bcanvas.width, this.bcanvas.height);
    }

    const max_dim = iw > ih ? iw : ih;
    const ow = Math.round((iw / max_dim) * MAX_DIM),
        oh = Math.round((ih / max_dim) * MAX_DIM);
        
    this.bcanvas.width = ow;
    this.bcanvas.height = oh;
    this.bctx.drawImage(image, 0, 0, iw, ih, 0, 0, ow, oh);
    return this.bctx.getImageData(0, 0, ow, oh);
}


(async () => {
    console.log('Starting...');
    console.log('--------------------------------------------');

    let retVal = [];
    let loggedMessages = [];
    const logger = console.log;
    const regex = /inlier_fraction\s+([\d.]+)/g;
    console.log = function(message) { // overwrite
        loggedMessages.push(message);
        // logger.apply(console, arguments);
    };
    const transformModule = await require('./app/js/transform');
    let mode = ['affine', 'perspective', 'tps', 'similarity', 'identity']; 
    for (let item of images) {
        const im1Data = await getImageData(item[0]);
        const im2Data = await getImageData(item[1]);
        transformModule.estimate_transform( im2Data.data, im2Data.height, im1Data.data, im1Data.height, mode[2] );
        // const H = new Float64Array( buf.buffer, buf.byteOffset, buf.length / Float64Array.BYTES_PER_ELEMENT ).slice();
        // const _warped_buf = transformModule.warp_image( im2Data.data, im2Data.height, H, im1Data.height, im1Data.width, mode[0] )
        // console.log("warped_buf", _warped_buf);
        // resultData = new ImageData( new Uint8ClampedArray(_warped_buf), im1Data.width, im1Data.height );
        // console.log("resultData", resultData);
        if (loggedMessages.length != 0) {
            let lastFraction = 0;
            let match;
            while ((match = regex.exec(loggedMessages.pop())) !== null) {
                lastFraction = parseFloat(match[1]) * 100;
            }
            retVal.push([item[0], item[1], lastFraction]);
        }
        loggedMessages = [];
    }
    console.log = logger;
    console.log(retVal);
    console.log('--------------------------------------------');
    console.log('Complete...');
    
})();




